# Generated by Django 4.1.7 on 2023-03-27 19:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appuser',
            name='created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created'),
        ),
    ]



from __future__ import annotations

import base64
import io
import os
from dataclasses import dataclass

import matplotlib
import matplotlib.dates as mdates
import matplotlib.font_manager as fm
import matplotlib.pyplot as plt
from asgiref.sync import sync_to_async
from matplotlib.ticker import LinearLocator

from django_insights.models import Bucket
from django_insights.settings import settings

matplotlib.use('Agg')
dir_path = os.path.dirname(os.path.realpath(__file__))
fm.fontManager.addfont(f"{dir_path}/static/insights/fonts/ubuntu.ttf")


plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['font.sans-serif'] = 'Ubuntu'


@dataclass
class ThemeColor:
    primary: str
    face: str
    tick: str
    grid: str
    edge: str


@dataclass
class Theme:
    dark: ThemeColor
    light: ThemeColor


themes = Theme(
    light=ThemeColor(
        primary=settings.INSIGHTS_CHART_LIGHT_PRIMARY_COLOR,
        face="#FEFEFE",
        tick="#333333",
        grid="#333333",
        edge="#FFFFFF",
    ),
    dark=ThemeColor(
        primary=settings.INSIGHTS_CHART_DARK_PRIMARY_COLOR,
        face="#0d1117",
        tick="#FEFEFE",
        grid="#FEFEFE",
        edge="#0d1117",
    ),
)


def to_bytes_io(fig) -> bytes:
    """Render Mathplotlib figure to file-like object"""
    flike = io.BytesIO()
    fig.savefig(flike, dpi=settings.INSIGHTS_CHART_DPI)

    return flike.getvalue()


def to_base64img(fig) -> str:
    """Render Mathplotlib figure to Base64 encoded image"""
    return base64.b64encode(to_bytes_io(fig)).decode()


def prepare_plot(bucket, theme):
    """Default plot options, used for all charts"""
    fig, ax = plt.subplots(figsize=(8, 4))
    fig.set_facecolor(theme.face)
    ax.tick_params(axis='x', colors=theme.tick)
    ax.tick_params(axis='y', colors=theme.tick)
    ax.set_facecolor(theme.face)
    ax.set_title(bucket.title)
    ax.set_ylabel(bucket.ylabel)
    ax.set_xlabel(bucket.xlabel)
    ax.yaxis.label.set_color(theme.tick)
    ax.xaxis.label.set_color(theme.tick)
    ax.grid(
        linestyle="dotted",
        linewidth=0.1,
        color=theme.grid,
        zorder=-10,
        visible=False,
    )

    for spine in ax.spines.values():
        spine.set_edgecolor(theme.edge)

    return fig, ax


def render_barchart(xaxis, yaxis, labels, bucket, theme) -> str:
    """Render barchart"""
    theme = getattr(themes, theme)
    fig, ax = prepare_plot(bucket, theme)
    ax.bar(labels, yaxis, color=theme.primary)

    return fig


def render_scatterplot(xaxis, yaxis, bucket, theme) -> str:
    """Render scatterplot"""
    theme = getattr(themes, theme)

    fig, ax = prepare_plot(bucket, theme)
    ax.scatter(xaxis, yaxis, color=theme.primary)

    return fig


def render_timeseries(xaxis, yaxis, bucket, theme) -> str:
    """Render timeseries"""
    theme = getattr(themes, theme)

    fig, ax = prepare_plot(bucket, theme)
    ax.plot(xaxis, yaxis, '--o', markersize=5, color=theme.primary)

    # Date formatting
    fig.autofmt_xdate()
    ax.fmt_xdata = mdates.DateFormatter(bucket.xformat)
    ax.yaxis.set_minor_locator(LinearLocator(25))

    return fig


@sync_to_async
def barchart(bucket: Bucket, theme: str) -> str:
    """Barchart chart"""
    values = bucket.values.all()
    yvalues = [bucket_value.yvalue for bucket_value in values]
    xvalues = [bucket_value.xvalue for bucket_value in values]
    labels = [bucket_value.category for bucket_value in values]

    return render_barchart(yvalues, xvalues, labels, bucket, theme)


@sync_to_async
def scatterplot(bucket: Bucket, theme: str) -> str:
    """Scatterplot chart"""
    values = bucket.values.all()
    yvalues = [bucket_value.yvalue for bucket_value in values]
    xvalues = [bucket_value.xvalue for bucket_value in values]

    return render_scatterplot(yvalues, xvalues, bucket, theme)


@sync_to_async
def timeseries(bucket: Bucket, theme: str) -> str:
    """Timeseries chart"""
    values = bucket.values.all()

    yvalues = [bucket_value.timestamp for bucket_value in values]
    xvalues = [bucket_value.xvalue for bucket_value in values]

    return render_timeseries(yvalues, xvalues, bucket, theme)



from django.core.management.base import BaseCommand

from django_insights.metrics import metrics
from django_insights.registry import registry


class Command(BaseCommand):

    """Show all registered audit events"""

    def handle(self, *args, **options):
        self.stdout.write(self.style.HTTP_INFO('[Django Insights] - Collect insights'))

        # Auto-discover
        registry.autodiscover_insights()

        # Save metrics
        metrics.collect()



import uuid

from django.db import models
from django.utils import timezone

from django_insights.choices import BucketType
from django_insights.database import database_entry
from django_insights.managers import BucketManager


class App(models.Model):
    """
    Apps that have insights

    """

    uuid = models.UUIDField(default=uuid.uuid4, unique=True, primary_key=True)
    module = models.CharField(max_length=254, db_index=True, unique=True)
    label = models.CharField(max_length=254, null=True)

    @property
    def name(self) -> str:
        return self.label or self.module


class ExecutionDelta(models.Model):
    """
    Delta is stored on every run and used for filtering, these values should never change

    """

    executed_at = models.DateTimeField(db_index=True, default=timezone.now)
    uuid = models.UUIDField(default=uuid.uuid4)


class MetricModel(models.Model):
    created_at = models.DateTimeField(db_index=True, default=timezone.now)
    label = models.CharField(max_length=254, db_index=True, unique=True)

    question = models.TextField(blank=True, null=True)
    desc = models.TextField(blank=True, null=True)

    class Meta:
        abstract = True


class Counter(MetricModel):
    """
    Simple counter, can incr or decr, label and app are unique

    """

    value = models.IntegerField(default=0)
    app = models.ForeignKey(App, related_name='counters', on_delete=models.CASCADE)

    class Meta:
        app_label = database_entry
        constraints = [
            models.UniqueConstraint(
                fields=['app', 'label'], name='counter_unique_app_label'
            )
        ]

    def __str__(self):
        return f"Insight Counter: {self.label} = {self.value}"


class Gauge(MetricModel):
    """
    Simple gauge, can be set or set with delta, label and app are unique

    """

    value = models.FloatField(default=float(0.0))
    app = models.ForeignKey(App, related_name='gauges', on_delete=models.CASCADE)

    def gauge(self, value=1, delta=False) -> int:
        """Set gauge value"""
        if not delta:
            self.value = value
        else:
            self.value = self.value + value

    class Meta:
        app_label = database_entry
        constraints = [
            models.UniqueConstraint(
                fields=['app', 'label'], name='gauge_unique_app_label'
            )
        ]

    def __str__(self):
        return f"Insight Gauge: {self.label} = {self.value}"


class Bucket(MetricModel):
    app = models.ForeignKey(App, related_name='buckets', on_delete=models.CASCADE)

    type = models.IntegerField(
        choices=BucketType.BUCKET_TYPES, default=BucketType.TIMESERIES
    )

    # Bucket specific plot options
    xlabel = models.CharField(max_length=254, blank=True, null=True)
    xformat = models.CharField(max_length=254, blank=True, null=True)
    ylabel = models.CharField(max_length=254, blank=True, null=True)
    yformat = models.CharField(max_length=254, blank=True, null=True)

    # title for plot
    title = models.CharField(max_length=254, blank=True, null=True)

    @property
    def is_timeseries(self) -> bool:
        return self.type == BucketType.TIMESERIES

    @property
    def is_histogram(self) -> bool:
        return self.type == BucketType.HISTOGRAM

    @property
    def is_scatterplot(self) -> bool:
        return self.type == BucketType.SCATTERPLOT

    @property
    def is_barchart(self) -> bool:
        return self.type == BucketType.BARCHART

    objects = BucketManager()

    class Meta:
        app_label = database_entry
        constraints = [
            models.UniqueConstraint(
                fields=[
                    'app',
                    'label',
                ],
                name='bucket_unique_app_label_timestamp',
            )
        ]


class BucketValue(models.Model):
    """
    Bucket values matrix: used for time-series, histogram and scatterpltos.
    Combination of timestamp, label and app are unique.

    """

    # Timestamp for timeseries
    timestamp = models.DateTimeField(db_index=True, blank=True, null=True)

    # Matrix
    xvalue = models.FloatField(default=float(0.0))
    yvalue = models.FloatField(default=float(0.0))
    category = models.CharField(db_index=True, max_length=254, null=True)

    bucket = models.ForeignKey(Bucket, related_name='values', on_delete=models.CASCADE)

    class Meta:
        app_label = database_entry
        constraints = [
            models.UniqueConstraint(
                fields=['bucket', 'timestamp'],
                name='bucketvalue_unique_bucket_timestamp',
            )
        ]

    def __str__(self):
        return (
            "Insight Bucket value: "
            f" timestamp={self.bucket.label}.{self.timestamp} x={self.xvalue} y={self.yvalue}"
        )



#!/usr/bin/env python
from __future__ import annotations

import os
import subprocess
import sys
from pathlib import Path

if __name__ == "__main__":
    os.chdir(Path(__file__).parent)
    os.environ["CUSTOM_COMPILE_COMMAND"] = "requirements/compile.py"
    os.environ["PIP_REQUIRE_VIRTUALENV"] = "0"
    common_args = [
        "-m",
        "piptools",
        "compile",
        "--generate-hashes",
        "--allow-unsafe",
    ] + sys.argv[1:]
    subprocess.run(
        [
            "python3.8",
            *common_args,
            "-P",
            "Django>=3.2a1,<3.3",
            "-o",
            "py38-django32.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.8",
            *common_args,
            "-P",
            "Django>=4.0a1,<4.1",
            "-o",
            "py38-django40.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.8",
            *common_args,
            "-P",
            "Django>=4.1a1,<4.2",
            "-o",
            "py38-django41.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.8",
            *common_args,
            "-P",
            "Django>=4.2a1,<5.0",
            "-o",
            "py38-django42.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.9",
            *common_args,
            "-P",
            "Django>=3.2a1,<3.3",
            "-o",
            "py39-django32.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.9",
            *common_args,
            "-P",
            "Django>=4.0a1,<4.1",
            "-o",
            "py39-django40.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.9",
            *common_args,
            "-P",
            "Django>=4.1a1,<4.2",
            "-o",
            "py39-django41.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.9",
            *common_args,
            "-P",
            "Django>=4.2a1,<5.0",
            "-o",
            "py39-django42.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.10",
            *common_args,
            "-P",
            "Django>=3.2a1,<3.3",
            "-o",
            "py310-django32.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.10",
            *common_args,
            "-P",
            "Django>=4.0a1,<4.1",
            "-o",
            "py310-django40.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.10",
            *common_args,
            "-P",
            "Django>=4.1a1,<4.2",
            "-o",
            "py310-django41.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.10",
            *common_args,
            "-P",
            "Django>=4.2a1,<5.0",
            "-o",
            "py310-django42.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.11",
            *common_args,
            "-P",
            "Django>=4.1a1,<4.2",
            "-o",
            "py311-django41.txt",
        ],
        check=True,
        capture_output=True,
    )
    subprocess.run(
        [
            "python3.11",
            *common_args,
            "-P",
            "Django>=4.2a1,<5.0",
            "-o",
            "py311-django42.txt",
        ],
        check=True,
        capture_output=True,
    )



from __future__ import annotations

from django_insights.metrics import metrics
from project.testapp.users.models import AppUser

label = "Users"


@metrics.counter(
    question="How many app users we have in our store?",
    desc="Number of app users",
)
def count_app_users() -> int:
    return AppUser.objects.count()



from __future__ import annotations

import os

import django

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

SECRET_KEY = "NOTASECRET"


DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "db/testapp.db"},
    "insights": {"ENGINE": "django.db.backends.sqlite3", "NAME": "db/insights.db"},
}

DATABASE_ROUTERS = ['django_insights.database.Router']

CACHES = {
    "default": {"BACKEND": "django.core.cache.backends.locmem.LocMemCache"},
}

ALLOWED_HOSTS: list[str] = []

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.staticfiles",
    "project.testapp",
    "project.testapp.users",
    "django_insights",
]

MIDDLEWARE_CLASSES = (
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

STATIC_URL = 'static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static/')
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

MEDIA_URL = 'media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media/')


DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

AUTH_USER_MODEL = "users.AppUser"

ROOT_URLCONF = "project.urls"
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True

if django.VERSION < (4, 0):
    USE_L10N = True

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

USE_TZ = True


# Django Insights settings

# Custom app name
INSIGHTS_APP_NAME = "Finq"

# Quality of chart images
INSIGHTS_CHART_DPI = 180

# Change primary color
INSIGHTS_CHART_LIGHT_PRIMARY_COLOR = "#2563EB"
INSIGHTS_CHART_DARK_PRIMARY_COLOR = "#BFDBFE"



from django.conf import settings
from django.conf.urls.static import static
from django.urls import include, path

urlpatterns = (
    [
        path(
            '',
            include('django_insights.urls', namespace='insights'),
        ),
    ]
    + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
)



from django import template
from django.urls import reverse
from django.utils.safestring import mark_safe

from django_insights.models import Bucket

register = template.Library()


@register.simple_tag
def chart(bucket: Bucket):
    img_url = reverse("insights:insight_chart", args=[bucket.pk])
    return mark_safe(
        f"""
        <img
            class="w-full h-auto border-gray-200 border-t-1"
            src="{img_url}"
            alt="{bucket.question}"
        />
    """
    )



from __future__ import annotations

from django.conf import settings as django_settings

__all__ = ["settings"]


class CustomSettings:
    """
    Django Insight settings

    """

    INSIGHTS_APP_NAME: str = "MyApp"
    INSIGHTS_CHART_DPI: int = 180
    INSIGHTS_DEFAULT_THEME = "dark"
    INSIGHTS_CHART_LIGHT_PRIMARY_COLOR = "#2563EB"
    INSIGHTS_CHART_DARK_PRIMARY_COLOR = "#BFDBFE"
    INSIGHT_CHARTS_USE_MEDIA_CACHE = False

    def __getattribute__(self, name):
        try:
            return getattr(django_settings, name)
        except AttributeError:
            return super().__getattribute__(name)


settings = CustomSettings()



from project.settings.base import *  # noqa

DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
    "insights": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
}



from __future__ import annotations

import functools
import importlib

from django_insights.choices import BucketType
from django_insights.metrics_types import (
    BarChartAnswer,
    BarChartType,
    CounterType,
    GaugeType,
    ScatterPlotAnswer,
    ScatterPlotType,
    TimeSeriesAnswer,
    TimeSeriesType,
)
from django_insights.models import App, Bucket, BucketValue, Counter, Gauge
from django_insights.registry import registry


class InsightMetrics:
    """Auto-generate metrics from multiple apps at once"""

    create_counters: list[Counter]
    create_gauges: list[Gauge]
    create_bucket_values: list[BucketValue]

    apps: dict[str, App] = {}

    def __init__(self, reset=True) -> None:
        self.create_counters = []
        self.create_gauges = []
        self.create_bucket_values = []
        self.delete_metrics() if reset else None

    def delete_metrics(self) -> None:
        """Reset current dataset if metrics are generated"""
        Counter.objects.all().delete()
        Gauge.objects.all().delete()
        Bucket.objects.all().delete()
        BucketValue.objects.all().delete()

    def get_memoized_app(self, module: str, label: str = None) -> App:
        """Memoize apps so we reduce query count"""
        if app := self.apps.get(module):
            return app

        # get or create new app
        app, _ = App.objects.get_or_create(module=module, defaults={'label': label})
        self.apps.update({module: app})

        return app

    def get_app(self, func) -> tuple[str, App]:
        """
        Get memoized app

        """

        label = func.__name__
        module = func.__module__
        insights_module = importlib.import_module(module)

        app_label = getattr(insights_module, 'label', None)
        app = self.get_memoized_app(module=module, label=app_label)

        return label, app

    def counter(self, question: str = None, desc: str = None):
        """
        Decorator to collect Counter metrics

        """

        def decorator(func):
            label, app = self.get_app(func)

            @functools.wraps(func)
            def inner(*args, **kwargs):
                counter_type = CounterType(value=func(*args, **kwargs))
                counter = Counter(
                    app=app,
                    label=label,
                    value=counter_type.value,
                    question=question,
                    desc=desc,
                )
                self.create_counters.append(counter)

            registry.register_insight(
                label=label,
                module=app.module,
                question=question,
                func=inner,
            )

            return None

        return decorator

    def gauge(self, question: str = None, desc: str = None):
        """
        Decorator to collect Gauge metric

        """

        def decorator(func):
            label, app = self.get_app(func)

            @functools.wraps(func)
            def inner(*args, **kwargs):
                gauge_type = GaugeType(value=func(*args, **kwargs))
                gauge = Gauge(
                    app=app,
                    label=label,
                    value=gauge_type.value,
                    question=question,
                    desc=desc,
                )
                self.create_gauges.append(gauge)

            registry.register_insight(
                label=label,
                module=app.module,
                question=question,
                func=inner,
            )

            return None

        return decorator

    def timeseries(
        self,
        question: str = None,
        desc: str = None,
        xlabel: str = None,
        xformat: str = None,
        ylabel: str = None,
        yformat: str = None,
        title=None,
    ):
        """
        Decorator to collect TimeSeries metrics

        """

        def decorator(func):
            label, app = self.get_app(func)

            @functools.wraps(func)
            def inner(*args, **kwargs):
                results = func(*args, **kwargs)
                ts_type = TimeSeriesType(
                    values=[TimeSeriesAnswer(*result) for result in results]
                )

                bucket = Bucket.objects.create(
                    app=app,
                    label=label,
                    question=question,
                    desc=desc,
                    xlabel=xlabel,
                    xformat=xformat,
                    ylabel=ylabel,
                    yformat=yformat,
                    title=title,
                    type=BucketType.TIMESERIES,
                )

                for row in ts_type.values:
                    bucket_value = BucketValue(
                        timestamp=row.timestamp, xvalue=row.xvalue, bucket=bucket
                    )
                    self.create_bucket_values.append(bucket_value)

            registry.register_insight(
                label=label,
                module=app.module,
                question=question,
                func=inner,
            )

            return None

        return decorator

    def scatterplot(
        self,
        question: str = None,
        desc: str = None,
        xlabel: str = None,
        xformat: str = None,
        ylabel: str = None,
        yformat: str = None,
        title=None,
    ):
        """
        Decorator to collect Scatterplot metrics

        """

        def decorator(func):
            label, app = self.get_app(func)

            @functools.wraps(func)
            def inner(*args, **kwargs):
                results = func(*args, **kwargs)
                scp_type = ScatterPlotType(
                    values=[ScatterPlotAnswer(*result) for result in results]
                )

                bucket = Bucket.objects.create(
                    app=app,
                    label=label,
                    question=question,
                    desc=desc,
                    xlabel=xlabel,
                    xformat=xformat,
                    ylabel=ylabel,
                    yformat=yformat,
                    title=title,
                    type=BucketType.SCATTERPLOT,
                )

                for row in scp_type.values:
                    bucket_value = BucketValue(
                        xvalue=row.xvalue,
                        yvalue=row.yvalue,
                        category=row.category,
                        bucket=bucket,
                    )
                    self.create_bucket_values.append(bucket_value)

            registry.register_insight(
                label=label,
                module=app.module,
                question=question,
                func=inner,
            )

            return None

        return decorator

    def barchart(
        self,
        question: str = None,
        desc: str = None,
        xlabel: str = None,
        xformat: str = None,
        ylabel: str = None,
        yformat: str = None,
        title=None,
    ):
        """
        Decorator to collect Barchart metrics

        """

        def decorator(func):
            label, app = self.get_app(func)

            @functools.wraps(func)
            def inner(*args, **kwargs):
                results = func(*args, **kwargs)
                bar_type = BarChartType(
                    values=[BarChartAnswer(*result) for result in results]
                )

                bucket = Bucket.objects.create(
                    app=app,
                    label=label,
                    question=question,
                    desc=desc,
                    xlabel=xlabel,
                    xformat=xformat,
                    ylabel=ylabel,
                    yformat=yformat,
                    title=title,
                    type=BucketType.BARCHART,
                )

                for row in bar_type.values:
                    bucket_value = BucketValue(
                        xvalue=row.xvalue,
                        yvalue=row.yvalue,
                        category=row.category,
                        bucket=bucket,
                    )
                    self.create_bucket_values.append(bucket_value)

            registry.register_insight(
                label=label,
                module=app.module,
                question=question,
                func=inner,
            )

            return None

        return decorator

    def collect(self):
        """
        Collect insights

        """
        registry.collect_insights()

        Counter.objects.bulk_create(self.create_counters)
        Gauge.objects.bulk_create(self.create_gauges)
        BucketValue.objects.bulk_create(self.create_bucket_values)


metrics = InsightMetrics()
